@@ -35,7 +35,7 @@
 class SPgSQLStatement: public SSqlStatement
 {
 public:
-  SPgSQLStatement(const string& query, bool dolog, int nparams, SPgSQL* db, unsigned int nstatement) {
+  SPgSQLStatement(const string& query, bool dolog, int nparams, SPgSQL* db) {
     d_query = query;
     d_dolog = dolog;
     d_parent = db;
@@ -45,7 +45,6 @@ public:
     d_res_set = NULL;
     paramValues = NULL;
     paramLengths = NULL;
-    d_nstatement = nstatement;
     d_paridx = 0;
     d_residx = 0;
     d_resnum = 0;
@@ -81,7 +80,7 @@ public:
       g_log<<Logger::Warning<< "Query "<<((long)(void*)this)<<": " << d_query << endl;
       d_dtime.set();
     }
-    d_res_set = PQexecPrepared(d_db(), d_stmt.c_str(), d_nparams, paramValues, paramLengths, NULL, 0);
+    d_res_set = PQexecParams(d_db(), d_query.c_str(), d_nparams, NULL, paramValues, paramLengths, NULL, 0);
     ExecStatusType status = PQresultStatus(d_res_set);
     if (status != PGRES_COMMAND_OK && status != PGRES_TUPLES_OK && status != PGRES_NONFATAL_ERROR) {
       string errmsg(PQresultErrorMessage(d_res_set));
@@ -150,12 +149,12 @@ public:
     row.reserve(PQnfields(d_res));
     for(i=0;i<PQnfields(d_res);i++) {
       if (PQgetisnull(d_res, d_residx, i)) {
-        row.push_back("");
+        row.emplace_back("");
       } else if (PQftype(d_res, i) == 16) { // BOOLEAN
         char *val = PQgetvalue(d_res, d_residx, i);
-        row.push_back(val[0] == 't' ? "1" : "0");
+        row.emplace_back(val[0] == 't' ? "1" : "0");
       } else {
-        row.push_back(string(PQgetvalue(d_res, d_residx, i)));
+        row.emplace_back(PQgetvalue(d_res, d_residx, i));
       }
     }
     d_residx++;
@@ -172,7 +171,7 @@ public:
     if (d_res == NULL) return this;
     result.reserve(d_resnum);
     row_t row;
-    while(hasNextRow()) { nextRow(row); result.push_back(row); }
+    while(hasNextRow()) { nextRow(row); result.push_back(std::move(row)); }
     return this;
   }
 
@@ -208,26 +207,10 @@ private:
   void releaseStatement() {
     d_prepared = false;
     reset();
-    if (!d_stmt.empty()) {
-      string cmd = string("DEALLOCATE " + d_stmt);
-      PGresult *res = PQexec(d_db(), cmd.c_str());
-      PQclear(res);
-      d_stmt.clear();
-    }
   }
 
   void prepareStatement() {
     if (d_prepared) return;
-    // prepare a statement; name must be unique per session (using d_nstatement to ensure this).
-    this->d_stmt = string("stmt") + std::to_string(d_nstatement);
-    PGresult* res = PQprepare(d_db(), d_stmt.c_str(), d_query.c_str(), d_nparams, NULL);
-    ExecStatusType status = PQresultStatus(res);
-    string errmsg(PQresultErrorMessage(res));
-    PQclear(res);
-    if (status != PGRES_COMMAND_OK && status != PGRES_TUPLES_OK && status != PGRES_NONFATAL_ERROR) {
-      releaseStatement();
-      throw SSqlException("Fatal error during prepare: " + d_query + string(": ") + errmsg);
-    }
     paramValues=NULL;
     d_cur_set=d_paridx=d_residx=d_resnum=d_fnum=0;
     paramLengths=NULL;
@@ -245,7 +228,6 @@ private:
   }
 
   string d_query;
-  string d_stmt;
   SPgSQL *d_parent;
   PGresult *d_res_set;
   PGresult *d_res;
@@ -260,7 +242,6 @@ private:
   int d_resnum;
   int d_fnum;
   int d_cur_set;
-  unsigned int d_nstatement;
 };
 
 bool SPgSQL::s_dolog;
@@ -271,7 +252,6 @@ SPgSQL::SPgSQL(const string &database, const string &host, const string& port, c
   d_db=0;
   d_in_trx = false;
   d_connectstr="";
-  d_nstatement = 0;
 
   if (!database.empty())
     d_connectstr+="dbname="+database;
@@ -338,8 +318,7 @@ void SPgSQL::execute(const string& query)
 
 std::unique_ptr<SSqlStatement> SPgSQL::prepare(const string& query, int nparams)
 {
-  d_nstatement++;
-  return std::unique_ptr<SSqlStatement>(new SPgSQLStatement(query, s_dolog, nparams, this, d_nstatement));
+  return std::unique_ptr<SSqlStatement>(new SPgSQLStatement(query, s_dolog, nparams, this));
 }
 
 void SPgSQL::startTransaction() {
