@@ -46,7 +46,10 @@
 
 #include "lmdbbackend.hh"
 
-#define SCHEMAVERSION 1
+#define SCHEMAVERSION 2
+#define SCHEMAVERSION_TEXT "2"
+// List the class version here. Default is 0
+BOOST_CLASS_VERSION(LMDBBackend::KeyDataDB, 1)
 
 LMDBBackend::LMDBBackend(const std::string& suffix)
 {
@@ -72,16 +75,19 @@ LMDBBackend::LMDBBackend(const std::string& suffix)
   
   auto pdnsdbi = d_tdomains->getEnv()->openDB("pdns", MDB_CREATE);
   auto txn = d_tdomains->getEnv()->getRWTransaction();
+  uint32_t schemaversion = 1;
   MDBOutVal _schemaversion;
   if(!txn->get(pdnsdbi, "schemaversion", _schemaversion)) {
-    auto schemaversion = _schemaversion.get<uint32_t>();
-    if (schemaversion != SCHEMAVERSION) {
+    schemaversion = _schemaversion.get<uint32_t>();
+  }
+
+  if (schemaversion != SCHEMAVERSION) {
+    if (getArgAsNum("schema-version") != SCHEMAVERSION) {
       throw std::runtime_error("Expected LMDB schema version "+std::to_string(SCHEMAVERSION)+" but got "+std::to_string(schemaversion));
     }
-  }
-  else {
     txn->put(pdnsdbi, "schemaversion", SCHEMAVERSION);
   }
+
   MDBOutVal shards;
   if(!txn->get(pdnsdbi, "shards", shards)) {
     
@@ -160,11 +166,23 @@ void serialize(Archive & ar, LMDBBackend::DomainMeta& g, const unsigned int vers
 }
 
 template<class Archive>
-void serialize(Archive & ar, LMDBBackend::KeyDataDB& g, const unsigned int version)
+void save(Archive & ar, const LMDBBackend::KeyDataDB& g, const unsigned int version)
+{
+  ar & g.domain & g.content & g.flags & g.active & g.published;
+}
+
+template<class Archive>
+void load(Archive & ar, LMDBBackend::KeyDataDB& g, const unsigned int version)
 {
   ar & g.domain & g.content & g.flags & g.active;
+  if (version >= 1) {
+    ar & g.published;
+  } else {
+    g.published = true;
+  }
 }
 
+
 template<class Archive>
 void serialize(Archive & ar, TSIGKey& g, const unsigned int version)
 {
@@ -180,6 +198,7 @@ void serialize(Archive & ar, TSIGKey& g, const unsigned int version)
 
 BOOST_SERIALIZATION_SPLIT_FREE(DNSName);
 BOOST_SERIALIZATION_SPLIT_FREE(QType);
+BOOST_SERIALIZATION_SPLIT_FREE(LMDBBackend::KeyDataDB);
 BOOST_IS_BITWISE_SERIALIZABLE(ComboAddress);
 
 template<>
@@ -518,11 +537,9 @@ bool LMDBBackend::deleteDomain(const DNSName &domain)
 
 bool LMDBBackend::list(const DNSName &target, int id, bool include_disabled)
 {
-  d_inlist=true;
   DomainInfo di;
   {
     auto dtxn = d_tdomains->getROTransaction();
-    
     if((di.id = dtxn.get<0>(target, di))) 
       ; //      cout<<"Found domain "<<target<<" on domain_id "<<di.id <<", list requested "<<id<<endl;
     else {
@@ -530,31 +547,30 @@ bool LMDBBackend::list(const DNSName &target, int id, bool include_disabled)
       return false;
     }
   }
-  
+
   d_rotxn = getRecordsROTransaction(di.id, d_rwtxn);
+  d_getcursor = std::make_shared<MDBROCursor>(d_rotxn->txn->getCursor(d_rotxn->db->dbi));
+
   compoundOrdername co;
   d_matchkey = co(di.id);
-  d_getcursor = std::make_shared<MDBROCursor>(d_rotxn->txn->getCursor(d_rotxn->db->dbi));
+
   MDBOutVal key, val;
-  d_inlist = true;
-  
   if(d_getcursor->lower_bound(d_matchkey, key, val) || key.get<StringView>().rfind(d_matchkey, 0) != 0) {
     // cout<<"Found nothing for list"<<endl;
     d_getcursor.reset();
-    return true;
   }
-  
-  d_lookupqname = target;
-  
+
+  d_lookupdomain = target;
   return true;
 }
 
-void LMDBBackend::lookup(const QType &type, const DNSName &qdomain, DNSPacket *p, int zoneId)
+void LMDBBackend::lookup(const QType &type, const DNSName &qdomain, int zoneId, DNSPacket *p)
 {
   if(d_dolog) {
     g_log << Logger::Warning << "Got lookup for "<<qdomain<<"|"<<type.getName()<<" in zone "<< zoneId<<endl;
     d_dtime.set();
   }
+
   DNSName hunt(qdomain);
   DomainInfo di;
   if(zoneId < 0) {
@@ -586,12 +602,11 @@ void LMDBBackend::lookup(const QType &type, const DNSName &qdomain, DNSPacket *p
   d_getcursor = std::make_shared<MDBROCursor>(d_rotxn->txn->getCursor(d_rotxn->db->dbi));
   MDBOutVal key, val;
   if(type.getCode() == QType::ANY) {
-    d_matchkey = co(zoneId,relqname);
+    d_matchkey = co(zoneId, relqname);
   }
   else {
-    d_matchkey= co(zoneId,relqname, type.getCode());
+    d_matchkey= co(zoneId, relqname, type.getCode());
   }
-  d_inlist=false;
   
   if(d_getcursor->lower_bound(d_matchkey, key, val) || key.get<StringView>().rfind(d_matchkey, 0) != 0) {
     d_getcursor.reset();
@@ -604,146 +619,68 @@ void LMDBBackend::lookup(const QType &type, const DNSName &qdomain, DNSPacket *p
   if(d_dolog) {
     g_log<<Logger::Warning<< "Query "<<((long)(void*)this)<<": "<<d_dtime.udiffNoReset()<<" usec to execute"<<endl;
   }
-    
-  d_lookuptype=type;
-  d_lookupqname = qdomain;
+
   d_lookupdomain = hunt;
-  d_lookupdomainid = zoneId;
 }
 
+
 bool LMDBBackend::get(DNSZoneRecord& rr)
-{
-  if(d_inlist)
-    return get_list(rr);
-  else
-    return get_lookup(rr);
-}
-
-bool LMDBBackend::get(DNSResourceRecord& rr)
-{
-  //  cout <<"Old-school get called"<<endl;
-  DNSZoneRecord dzr;
-  if(d_inlist) {
-    if(!get_list(dzr))
-      return false;
-  }
-  else {
-    if(!get_lookup(dzr))
-      return false;
-  }
-  rr.qname = dzr.dr.d_name;
-  rr.ttl = dzr.dr.d_ttl;
-  rr.qtype =dzr.dr.d_type;
-  rr.content = dzr.dr.d_content->getZoneRepresentation(true);
-  rr.domain_id = dzr.domain_id;
-  rr.auth = dzr.auth;
-  //  cout<<"old school called for "<<rr.qname<<", "<<rr.qtype.getName()<<endl;
-  return true;
-}
-
-bool LMDBBackend::getSOA(const DNSName &domain, SOAData &sd)
-{
-  //  cout <<"Native getSOA called"<<endl;
-  lookup(QType(QType::SOA), domain, nullptr, -1);
-  DNSZoneRecord dzr;
-  bool found=false;
-  while(get(dzr)) {
-    auto src = getRR<SOARecordContent>(dzr.dr);
-    sd.domain_id = dzr.domain_id;
-    sd.ttl = dzr.dr.d_ttl;
-    sd.qname = dzr.dr.d_name;
-    
-    sd.nameserver = src->d_mname;
-    sd.hostmaster = src->d_rname;
-    sd.serial = src->d_st.serial;
-    sd.refresh = src->d_st.refresh;
-    sd.retry = src->d_st.retry;
-    sd.expire = src->d_st.expire;
-    sd.default_ttl = src->d_st.minimum;
-    
-    sd.db = this;
-    found=true;
-  }
-  return found;
-}
-bool LMDBBackend::get_list(DNSZoneRecord& rr)
-{
-  for(;;) {
-    if(!d_getcursor)  {
-      d_rotxn.reset();
-      return false;
-    }
-    
-    MDBOutVal keyv, val;
-
-    d_getcursor->current(keyv, val);
-    DNSResourceRecord drr;
-    serFromString(val.get<string>(), drr);
-  
-    auto key = keyv.get<string_view>();
-    rr.dr.d_name = compoundOrdername::getQName(key) + d_lookupqname;
-    rr.domain_id = compoundOrdername::getDomainID(key);
-    rr.dr.d_type = compoundOrdername::getQType(key).getCode();
-    rr.dr.d_ttl = drr.ttl;
-    rr.auth = drr.auth;
-  
-    if(rr.dr.d_type == QType::NSEC3) {
-      //      cout << "Had a magic NSEC3, skipping it" << endl;
-      if(d_getcursor->next(keyv, val) || keyv.get<StringView>().rfind(d_matchkey, 0) != 0) {
-        d_getcursor.reset();
-      }
-      continue;
-    }
-    rr.dr.d_content = unserializeContentZR(rr.dr.d_type, rr.dr.d_name, drr.content);
-    
-    if(d_getcursor->next(keyv, val) || keyv.get<StringView>().rfind(d_matchkey, 0) != 0) {
-      d_getcursor.reset();
-    }
-    break;
-  }
-  return true;
-}
-
-
-bool LMDBBackend::get_lookup(DNSZoneRecord& rr)
 {
   for(;;) {
     if(!d_getcursor) {
       d_rotxn.reset();
       return false;
     }
+
     MDBOutVal keyv, val;
     d_getcursor->current(keyv, val);
+
     DNSResourceRecord drr;
     serFromString(val.get<string>(), drr);
-    
+
     auto key = keyv.get<string_view>();
-    
-    rr.dr.d_name = compoundOrdername::getQName(key) + d_lookupdomain;
-    
-    rr.domain_id = compoundOrdername::getDomainID(key);
-    //  cout << "We found "<<rr.qname<< " in zone id "<<rr.domain_id <<endl;
     rr.dr.d_type = compoundOrdername::getQType(key).getCode();
-    rr.dr.d_ttl = drr.ttl;
+
     if(rr.dr.d_type == QType::NSEC3) {
-      //      cout << "Hit a magic NSEC3 skipping" << endl;
+      // Hit a magic NSEC3 skipping
       if(d_getcursor->next(keyv, val) || keyv.get<StringView>().rfind(d_matchkey, 0) != 0) {
         d_getcursor.reset();
-        d_rotxn.reset();
       }
+
       continue;
     }
-    
+
+    rr.dr.d_name = compoundOrdername::getQName(key) + d_lookupdomain;
+    rr.domain_id = compoundOrdername::getDomainID(key);
+    rr.dr.d_ttl = drr.ttl;
     rr.dr.d_content = unserializeContentZR(rr.dr.d_type, rr.dr.d_name, drr.content);
     rr.auth = drr.auth;
+
     if(d_getcursor->next(keyv, val) || keyv.get<StringView>().rfind(d_matchkey, 0) != 0) {
       d_getcursor.reset();
-      d_rotxn.reset();
     }
+
     break;
   }
 
-  
+  return true;
+}
+
+
+bool LMDBBackend::get(DNSResourceRecord& rr)
+{
+  DNSZoneRecord dzr;
+  if(!get(dzr)) {
+    return false;
+  }
+
+  rr.qname = dzr.dr.d_name;
+  rr.ttl = dzr.dr.d_ttl;
+  rr.qtype = dzr.dr.d_type;
+  rr.content = dzr.dr.d_content->getZoneRepresentation(true);
+  rr.domain_id = dzr.domain_id;
+  rr.auth = dzr.auth;
+
   return true;
 }
 
@@ -969,7 +906,7 @@ bool LMDBBackend::getDomainKeys(const DNSName& name, std::vector<KeyData>& keys)
   auto txn = d_tkdb->getROTransaction();
   auto range = txn.equal_range<0>(name);
   for(auto& iter = range.first; iter != range.second; ++iter) {
-    KeyData kd{iter->content, iter.getID(), iter->flags, iter->active};
+    KeyData kd{iter->content, iter.getID(), iter->flags, iter->active, iter->published};
     keys.push_back(kd);
   }
 
@@ -994,7 +931,7 @@ bool LMDBBackend::removeDomainKey(const DNSName& name, unsigned int id)
 bool LMDBBackend::addDomainKey(const DNSName& name, const KeyData& key, int64_t& id)
 {
   auto txn = d_tkdb->getRWTransaction();
-  KeyDataDB kdb{name, key.content, key.flags, key.active};
+  KeyDataDB kdb{name, key.content, key.flags, key.active, key.published};
   id = txn.put(kdb);
   txn.commit();
     
@@ -1034,10 +971,48 @@ bool LMDBBackend::deactivateDomainKey(const DNSName& name, unsigned int id)
       return true;
     }
   }
-  // cout << "??? wanted to activate domain key for domain "<<name<<" with id "<<id<<", could not find it"<<endl;
+  // cout << "??? wanted to deactivate domain key for domain "<<name<<" with id "<<id<<", could not find it"<<endl;
   return true;
 }
 
+bool LMDBBackend::publishDomainKey(const DNSName& name, unsigned int id)
+{
+  auto txn = d_tkdb->getRWTransaction();
+  KeyDataDB kdb;
+  if(txn.get(id, kdb)) {
+    if(kdb.domain == name) {
+      txn.modify(id, [](KeyDataDB& kdbarg)
+                 {
+                   kdbarg.published = true;
+                 });
+      txn.commit();
+      return true;
+    }
+  }
+
+  // cout << "??? wanted to hide domain key for domain "<<name<<" with id "<<id<<", could not find it"<<endl;
+  return true;
+}
+
+bool LMDBBackend::unpublishDomainKey(const DNSName& name, unsigned int id)
+{
+  auto txn = d_tkdb->getRWTransaction();
+  KeyDataDB kdb;
+  if(txn.get(id, kdb)) {
+    if(kdb.domain == name) {
+      txn.modify(id, [](KeyDataDB& kdbarg)
+                 {
+                   kdbarg.published = false;
+                 });
+      txn.commit();
+      return true;
+    }
+  }
+  // cout << "??? wanted to unhide domain key for domain "<<name<<" with id "<<id<<", could not find it"<<endl;
+  return true;
+}
+
+
 bool LMDBBackend::getBeforeAndAfterNamesAbsolute(uint32_t id, const DNSName& qname, DNSName& unhashed, DNSName& before, DNSName& after) 
 {
   //  cout << __PRETTY_FUNCTION__<< ": "<<id <<", "<<qname << " " << unhashed<<endl;
@@ -1605,7 +1580,8 @@ public:
     declare(suffix,"filename","Filename for lmdb","./pdns.lmdb");
     declare(suffix,"sync-mode","Synchronisation mode: nosync, nometasync, mapasync, sync","mapasync");
     // there just is no room for more on 32 bit
-    declare(suffix,"shards","Records database will be split into this number of shards", (sizeof(long) == 4) ? "2" : "64"); 
+    declare(suffix,"shards","Records database will be split into this number of shards", (sizeof(long) == 4) ? "2" : "64");
+    declare(suffix,"schema-version","Maximum allowed schema version to run on this DB. If a lower version is found, auto update is performed", SCHEMAVERSION_TEXT); 
   }
   DNSBackend *make(const string &suffix="")
   {
