@@ -55,7 +55,7 @@ static po::variables_map g_vm;
 
 static bool g_quiet;
 
-static void* recvThread(const vector<Socket*>* sockets)
+static void* recvThread(const vector<std::unique_ptr<Socket>>* sockets)
 {
   vector<pollfd> rfds, fds;
   for(const auto& s : *sockets) {
@@ -71,11 +71,13 @@ static void* recvThread(const vector<Socket*>* sockets)
 #if HAVE_RECVMMSG
   vector<struct mmsghdr> buf(100);
   for(auto& m : buf) {
-    fillMSGHdr(&m.msg_hdr, new struct iovec, new char[512], 512, new char[1500], 1500, new ComboAddress("127.0.0.1"));
+    cmsgbuf_aligned *cbuf = new cmsgbuf_aligned;
+    fillMSGHdr(&m.msg_hdr, new struct iovec, cbuf, sizeof(*cbuf), new char[1500], 1500, new ComboAddress("127.0.0.1"));
   }
 #else
   struct msghdr buf;
-  fillMSGHdr(&buf, new struct iovec, new char[512], 512, new char[1500], 1500, new ComboAddress("127.0.0.1"));
+  cmsgbuf_aligned *cbuf = new cmsgbuf_aligned;
+  fillMSGHdr(&buf, new struct iovec, cbuf, sizeof(*cbuf), new char[1500], 1500, new ComboAddress("127.0.0.1"));
 #endif
 
   while(!g_done) {
@@ -106,7 +108,7 @@ static void* recvThread(const vector<Socket*>* sockets)
           continue;
         }
         g_recvcounter++;
-        for (int i = 0; i < buf.msg_iovlen; i++)
+        for (unsigned int i = 0; i < buf.msg_iovlen; i++)
           g_recvbytes += buf.msg_iov[i].iov_len;
 #endif
       }
@@ -129,7 +131,7 @@ static void setSocketBuffer(int fd, int optname, uint32_t size)
 
   if (setsockopt(fd, SOL_SOCKET, optname, (char*)&size, sizeof(size)) < 0 ) {
     if (!g_quiet) {
-      cerr<<"Warning: unable to raise socket buffer size to "<<size<<": "<<strerror(errno)<<endl;
+      cerr<<"Warning: unable to raise socket buffer size to "<<size<<": "<<stringerror()<<endl;
     }
   }
 }
@@ -168,7 +170,7 @@ static void replaceEDNSClientSubnet(vector<uint8_t>* packet, const Netmask& ecsR
   memcpy(&packet->at(packetSize - sizeof(addr)), &addr, sizeof(addr));
 }
 
-static void sendPackets(const vector<Socket*>* sockets, const vector<vector<uint8_t>* >& packets, int qps, ComboAddress dest, const Netmask& ecsRange)
+static void sendPackets(const vector<std::unique_ptr<Socket>>& sockets, const vector<vector<uint8_t>* >& packets, int qps, ComboAddress dest, const Netmask& ecsRange)
 {
   unsigned int burst=100;
   const auto nsecPerBurst=1*(unsigned long)(burst*1000000000.0/qps);
@@ -183,7 +185,7 @@ static void sendPackets(const vector<Socket*>* sockets, const vector<vector<uint
   struct Unit {
     struct msghdr msgh;
     struct iovec iov;
-    char cbuf[256];
+    cmsgbuf_aligned cbuf;
   };
   vector<unique_ptr<Unit> > units;
   int ret;
@@ -198,7 +200,7 @@ static void sendPackets(const vector<Socket*>* sockets, const vector<vector<uint
     }
 
     fillMSGHdr(&u.msgh, &u.iov, nullptr, 0, (char*)&(*p)[0], p->size(), &dest);
-    if((ret=sendmsg((*sockets)[count % sockets->size()]->getHandle(), 
+    if((ret=sendmsg(sockets[count % sockets.size()]->getHandle(), 
 		    &u.msgh, 0)))
       if(ret < 0)
 	      unixDie("sendmsg");
@@ -333,7 +335,7 @@ try
       ecsRange = Netmask(g_vm["ecs"].as<string>());
       if (!ecsRange.empty()) {
 
-        if (!ecsRange.isIpv4()) {
+        if (!ecsRange.isIPv4()) {
           cerr<<"Only IPv4 ranges are supported for ECS at the moment!"<<endl;
           return EXIT_FAILURE;
         }
@@ -355,7 +357,7 @@ try
 #if HAVE_SCHED_SETSCHEDULER
   if(sched_setscheduler(0, SCHED_FIFO, &param) < 0) {
     if (!g_quiet) {
-      cerr<<"Unable to set SCHED_FIFO: "<<strerror(errno)<<endl;
+      cerr<<"Unable to set SCHED_FIFO: "<<stringerror()<<endl;
     }
   }
 #endif
@@ -392,7 +394,7 @@ try
 
     DNSPacketWriter pw(packet, DNSName(qname), DNSRecordContent::TypeToNumber(qtype));
     pw.getHeader()->rd=wantRecursion;
-    pw.getHeader()->id=dns_random(UINT16_MAX);
+    pw.getHeader()->id=dns_random_uint16();
 
     if(!subnet.empty() || !ecsRange.empty()) {
       EDNSSubnetOpts opt;
@@ -411,7 +413,7 @@ try
     cout<<"Generated "<<unknown.size()<<" ready to use queries"<<endl;
   }
   
-  vector<Socket*> sockets;
+  vector<std::unique_ptr<Socket>> sockets;
   ComboAddress dest;
   try {
     dest = ComboAddress(g_vm["destination"].as<string>(), 53);
@@ -421,11 +423,11 @@ try
     return EXIT_FAILURE;
   }
   for(int i=0; i < 24; ++i) {
-    Socket *sock = new Socket(dest.sin4.sin_family, SOCK_DGRAM);
+    auto sock = make_unique<Socket>(dest.sin4.sin_family, SOCK_DGRAM);
     //    sock->connect(dest);
     setSocketSendBuffer(sock->getHandle(), 2000000);
     setSocketReceiveBuffer(sock->getHandle(), 2000000);
-    sockets.push_back(sock);
+    sockets.push_back(std::move(sock));
   }
   new thread(recvThread, &sockets);
   uint32_t qps;
@@ -434,7 +436,7 @@ try
   if (g_vm.count("plot-file")) {
     plot.open(g_vm["plot-file"].as<string>());
     if (!plot) {
-      cerr<<"Error opening "<<g_vm["plot-file"].as<string>()<<" for writing: "<<strerror(errno)<<endl;
+      cerr<<"Error opening "<<g_vm["plot-file"].as<string>()<<" for writing: "<<stringerror()<<endl;
       return EXIT_FAILURE;
     }
   }
@@ -477,7 +479,7 @@ try
     DTime dt;
     dt.set();
 
-    sendPackets(&sockets, toSend, qps, dest, ecsRange);
+    sendPackets(sockets, toSend, qps, dest, ecsRange);
     
     const auto udiff = dt.udiffNoReset();
     const auto realqps=toSend.size()/(udiff/1000000.0);
