@@ -66,7 +66,7 @@ try {
 
   FILE* fp = fopen(argv[2], "w");
   if (!fp) {
-    cerr<<"Error opening output file "<<argv[2]<<": "<<strerror(errno)<<endl;
+    cerr<<"Error opening output file "<<argv[2]<<": "<<stringerror()<<endl;
     exit(EXIT_FAILURE);
   }
 
