@@ -191,7 +191,7 @@ void OPTRecordContent::getData(vector<pair<uint16_t, string> >& options)
 
     string field(d_data.c_str() + pos, len);
     pos+=len;
-    options.push_back(make_pair(code, field));
+    options.push_back(make_pair(code, std::move(field)));
   }
 }
 
