@@ -212,7 +212,7 @@ try
   typedef map<uint16_t,uint32_t> rcodes_t;
   rcodes_t rcodes;
   
-  time_t lowestTime=2000000000, highestTime=0;
+  time_t lowestTime=0, highestTime=0;
   time_t lastsec=0;
   LiveCounts lastcounts;
   std::unordered_set<ComboAddress, ComboAddress::addressOnlyHash> requestors, recipients, rdnonra;
@@ -222,9 +222,9 @@ try
 
   for(unsigned int fno=0; fno < files.size(); ++fno) {
     PcapPacketReader pr(files[fno]);
-    PcapPacketWriter* pw=0;
+    std::unique_ptr<PcapPacketWriter> pw=nullptr;
     if(!g_vm["write-failures"].as<string>().empty())
-      pw=new PcapPacketWriter(g_vm["write-failures"].as<string>(), pr);
+      pw=std::unique_ptr<PcapPacketWriter>(new PcapPacketWriter(g_vm["write-failures"].as<string>(), pr));
  
     EDNSOpts edo;
     while(pr.getUDPPacket()) {
@@ -290,7 +290,8 @@ try
 	    lastcounts = lc;
 	  }
 
-	  lowestTime=min((time_t)lowestTime,  (time_t)pr.d_pheader.ts.tv_sec);
+    if(lowestTime) { lowestTime = min((time_t)lowestTime,  (time_t)pr.d_pheader.ts.tv_sec); }
+    else { lowestTime = pr.d_pheader.ts.tv_sec; }
 	  highestTime=max((time_t)highestTime, (time_t)pr.d_pheader.ts.tv_sec);
 
 	  QuestionIdentifier qi=QuestionIdentifier::create(pr.getSource(), pr.getDest(), header, qname, qtype);
@@ -359,7 +360,7 @@ try
 	      rem.sin4.sin_port=0;
 
 	      if(doServFailTree)
-		root.submit(qname, header.rcode, rem);
+		root.submit(qname, header.rcode, pr.d_len, rem);
 	    }
 
 	    if(!qd.d_qcount || qd.d_qcount == qd.d_answercount) {
