@@ -196,7 +196,7 @@ bool DNSCryptoKeyEngine::testAll()
         
         for(maker_t* verifier :  value.second) {
           try {
-            /* pair<unsigned int, unsigned int> res=*/ testMakers(value.first, creator, signer, verifier);
+            testMakers(value.first, creator, signer, verifier);
           }
           catch(std::exception& e)
           {
@@ -221,7 +221,7 @@ bool DNSCryptoKeyEngine::testOne(int algo)
 
       for(maker_t* verifier :  getAllMakers()[algo]) {
         try {
-          /* pair<unsigned int, unsigned int> res=*/testMakers(algo, creator, signer, verifier);
+          testMakers(algo, creator, signer, verifier);
         }
         catch(std::exception& e)
         {
@@ -233,8 +233,8 @@ bool DNSCryptoKeyEngine::testOne(int algo)
   }
   return ret;
 }
-// returns times it took to sign and verify
-pair<unsigned int, unsigned int> DNSCryptoKeyEngine::testMakers(unsigned int algo, maker_t* creator, maker_t* signer, maker_t* verifier)
+
+void DNSCryptoKeyEngine::testMakers(unsigned int algo, maker_t* creator, maker_t* signer, maker_t* verifier)
 {
   shared_ptr<DNSCryptoKeyEngine> dckeCreate(creator(algo));
   shared_ptr<DNSCryptoKeyEngine> dckeSign(signer(algo));
@@ -253,7 +253,11 @@ pair<unsigned int, unsigned int> DNSCryptoKeyEngine::testMakers(unsigned int alg
   else
     throw runtime_error("Can't guess key size for algorithm "+std::to_string(algo));
 
-  dckeCreate->create(bits);
+  DTime dt; dt.set();
+  for(unsigned int n = 0; n < 100; ++n)
+    dckeCreate->create(bits);
+  cerr<<"("<<dckeCreate->getBits()<<" bits) ";
+  unsigned int udiffCreate = dt.udiff() / 100;
 
   { // FIXME: this block copy/pasted from makeFromISCString
     DNSKEYRecordContent dkrc;
@@ -298,7 +302,7 @@ pair<unsigned int, unsigned int> DNSCryptoKeyEngine::testMakers(unsigned int alg
   string message("Hi! How is life?");
   
   string signature;
-  DTime dt; dt.set();
+  dt.set();
   for(unsigned int n = 0; n < 100; ++n)
     signature = dckeSign->sign(message);
   unsigned int udiffSign= dt.udiff()/100, udiffVerify;
@@ -308,14 +312,17 @@ pair<unsigned int, unsigned int> DNSCryptoKeyEngine::testMakers(unsigned int alg
     throw runtime_error("Comparison of public key loaded into verifier produced by signer failed");
   }
   dt.set();
-  if(dckeVerify->verify(message, signature)) {
-    udiffVerify = dt.udiff();
-    cerr<<"Signature & verify ok, signature "<<udiffSign<<"usec, verify "<<udiffVerify<<"usec"<<endl;
+  bool verified;
+  for(unsigned int n = 0; n < 100; ++n)
+    verified = dckeVerify->verify(message, signature);
+
+  if(verified) {
+    udiffVerify = dt.udiff() / 100;
+    cerr<<"Signature & verify ok, create "<<udiffCreate<<"usec, signature "<<udiffSign<<"usec, verify "<<udiffVerify<<"usec"<<endl;
   }
   else {
     throw runtime_error("Verification of creator "+dckeCreate->getName()+" with signer "+dckeSign->getName()+" and verifier "+dckeVerify->getName()+" failed");
   }
-  return make_pair(udiffSign, udiffVerify);
 }
 
 shared_ptr<DNSCryptoKeyEngine> DNSCryptoKeyEngine::makeFromPublicKeyString(unsigned int algorithm, const std::string& content)
@@ -344,12 +351,6 @@ shared_ptr<DNSCryptoKeyEngine> DNSCryptoKeyEngine::makeFromPEMString(DNSKEYRecor
   return 0;
 }
 
-
-static bool sharedDNSSECCompare(const shared_ptr<DNSRecordContent>& a, const shared_ptr<DNSRecordContent>& b)
-{
-  return a->serialize(g_rootdnsname, true, true) < b->serialize(g_rootdnsname, true, true);
-}
-
 /**
  * Returns the string that should be hashed to create/verify the RRSIG content
  *
@@ -363,10 +364,8 @@ static bool sharedDNSSECCompare(const shared_ptr<DNSRecordContent>& a, const sha
  *                            purposes, as the authoritative server correctly
  *                            sets qname to the wildcard.
  */
-string getMessageForRRSET(const DNSName& qname, const RRSIGRecordContent& rrc, vector<shared_ptr<DNSRecordContent> >& signRecords, bool processRRSIGLabels)
+string getMessageForRRSET(const DNSName& qname, const RRSIGRecordContent& rrc, const sortedRecords_t& signRecords, bool processRRSIGLabels)
 {
-  sort(signRecords.begin(), signRecords.end(), sharedDNSSECCompare);
-
   string toHash;
   toHash.append(const_cast<RRSIGRecordContent&>(rrc).serialize(g_rootdnsname, true, true));
   toHash.resize(toHash.size() - rrc.d_signature.length()); // chop off the end, don't sign the signature!
@@ -389,7 +388,7 @@ string getMessageForRRSET(const DNSName& qname, const RRSIGRecordContent& rrc, v
     }
   }
 
-  for(shared_ptr<DNSRecordContent>& add :  signRecords) {
+  for(const shared_ptr<DNSRecordContent>& add : signRecords) {
     toHash.append(nameToHash);
     uint16_t tmp=htons(rrc.d_type);
     toHash.append((char*)&tmp, 2);
@@ -417,13 +416,13 @@ bool DNSCryptoKeyEngine::isAlgorithmSupported(unsigned int algo)
 static unsigned int digestToAlgorithmNumber(uint8_t digest)
 {
   switch(digest) {
-  case DNSSECKeeper::SHA1:
+  case DNSSECKeeper::DIGEST_SHA1:
     return DNSSECKeeper::RSASHA1;
-  case DNSSECKeeper::SHA256:
+  case DNSSECKeeper::DIGEST_SHA256:
     return DNSSECKeeper::RSASHA256;
-  case DNSSECKeeper::GOST:
+  case DNSSECKeeper::DIGEST_GOST:
     return DNSSECKeeper::ECCGOST;
-  case DNSSECKeeper::SHA384:
+  case DNSSECKeeper::DIGEST_SHA384:
     return DNSSECKeeper::ECDSA384;
   default:
     throw std::runtime_error("Unknown digest type " + std::to_string(digest));
