@@ -86,42 +86,51 @@ extern StatBag S;
 
 vector<ComboAddress> g_localaddresses; // not static, our unit tests need to poke this
 
-void UDPNameserver::bindIPv4()
+void UDPNameserver::bindAddresses()
 {
   vector<string>locals;
+  stringtok(locals,::arg()["local-ipv6"]," ,");
+  if (!locals.empty()) {
+    g_log<<Logger::Error<<"NOTE: Deprecated local-ipv6 setting used. Please move those addresses to the local-address setting."<<endl;
+  }
   stringtok(locals,::arg()["local-address"]," ,");
+
   int one = 1;
 
   if(locals.empty())
     throw PDNSException("No local address specified");
 
   int s;
-  for(vector<string>::const_iterator i=locals.begin();i!=locals.end();++i) {
-    string localname(*i);
-    ComboAddress locala;
+  // for(vector<string>::const_iterator i=locals.begin();i!=locals.end();++i) {
+  for (const auto &local : locals) {
+    ComboAddress locala(local, ::arg().asNum("local-port"));
 
-    s=socket(AF_INET,SOCK_DGRAM,0);
+    s = socket(locala.sin4.sin_family, SOCK_DGRAM, 0);
 
-    if(s<0) {
-      g_log<<Logger::Error<<"Unable to acquire UDP socket: "+string(strerror(errno)) << endl;
-      throw PDNSException("Unable to acquire a UDP socket: "+string(strerror(errno)));
+    if(s < 0) {
+      if(errno == EAFNOSUPPORT) {
+        g_log<<Logger::Error<<"Binding "<<locala.toStringWithPort()<<": Address Family is not supported - skipping bind" << endl;
+        return;
+      }
+      throw PDNSException("Unable to acquire a UDP socket: "+stringerror());
     }
-  
+
     setCloseOnExec(s);
-  
     if(!setNonBlocking(s))
-      throw PDNSException("Unable to set UDP socket to non-blocking: "+stringerror());
+      throw PDNSException("Unable to set UDP socket " + locala.toStringWithPort() + " to non-blocking: "+stringerror());
 
-    locala=ComboAddress(localname, ::arg().asNum("local-port"));
-
-    if(locala.sin4.sin_family != AF_INET)
-      throw PDNSException("Attempting to bind IPv4 socket to IPv6 address");
-
-    if(IsAnyAddress(locala))
+    if(IsAnyAddress(locala)) {
       setsockopt(s, IPPROTO_IP, GEN_IP_PKTINFO, &one, sizeof(one));
+      if (locala.isIPv6()) {
+        setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, &one, sizeof(one));      // if this fails, we report an error in tcpreceiver too
+#ifdef IPV6_RECVPKTINFO
+        setsockopt(s, IPPROTO_IPV6, IPV6_RECVPKTINFO, &one, sizeof(one));
+#endif
+      }
+    }
 
     if (!setSocketTimestamps(s))
-      g_log<<Logger::Warning<<"Unable to enable timestamp reporting for socket"<<endl;
+      g_log<<Logger::Warning<<"Unable to enable timestamp reporting for socket "<<locala.toStringWithPort()<<endl;
 
     if (locala.isIPv4()) {
       try {
@@ -139,17 +148,16 @@ void UDPNameserver::bindIPv4()
 #endif
 
     if( ::arg().mustDo("non-local-bind") )
-	Utility::setBindAny(AF_INET, s);
-
+      Utility::setBindAny(locala.sin4.sin_family, s);
 
     if( !d_additional_socket )
         g_localaddresses.push_back(locala);
 
     if(::bind(s, (sockaddr*)&locala, locala.getSocklen()) < 0) {
-      string binderror = strerror(errno);
+      string binderror = stringerror();
       close(s);
       if( errno == EADDRNOTAVAIL && ! ::arg().mustDo("local-address-nonexist-fail") ) {
-        g_log<<Logger::Error<<"IPv4 Address " << localname << " does not exist on this server - skipping UDP bind" << endl;
+        g_log<<Logger::Error<<"Address " << locala << " does not exist on this server - skipping UDP bind" << endl;
         continue;
       } else {
         g_log<<Logger::Error<<"Unable to bind UDP socket to '"+locala.toStringWithPort()+"': "<<binderror<<endl;
@@ -157,12 +165,12 @@ void UDPNameserver::bindIPv4()
       }
     }
     d_sockets.push_back(s);
-    g_log<<Logger::Error<<"UDP server bound to "<<locala.toStringWithPort()<<endl;
     struct pollfd pfd;
     pfd.fd = s;
     pfd.events = POLLIN;
     pfd.revents = 0;
     d_rfds.push_back(pfd);
+    g_log<<Logger::Error<<"UDP server bound to "<<locala.toStringWithPort()<<endl;
   }
 }
 
@@ -198,79 +206,6 @@ bool AddressIsUs(const ComboAddress& remote)
   return false;
 }
 
-
-void UDPNameserver::bindIPv6()
-{
-  vector<string> locals;
-  stringtok(locals,::arg()["local-ipv6"]," ,");
-  int one=1;
-
-  if(locals.empty())
-    return;
-
-  int s;
-  for(vector<string>::const_iterator i=locals.begin();i!=locals.end();++i) {
-    string localname(*i);
-
-    s=socket(AF_INET6,SOCK_DGRAM,0);
-    if(s<0) {
-      if( errno == EAFNOSUPPORT ) {
-        g_log<<Logger::Error<<"IPv6 Address Family is not supported - skipping UDPv6 bind" << endl;
-        return;
-      } else {
-        g_log<<Logger::Error<<"Unable to acquire a UDPv6 socket: "+string(strerror(errno)) << endl;
-        throw PDNSException("Unable to acquire a UDPv6 socket: "+string(strerror(errno)));
-      }
-    }
-
-    setCloseOnExec(s);
-    if(!setNonBlocking(s))
-      throw PDNSException("Unable to set UDPv6 socket to non-blocking: "+stringerror());
-
-    ComboAddress locala(localname, ::arg().asNum("local-port"));
-    
-    if(IsAnyAddress(locala)) {
-      setsockopt(s, IPPROTO_IP, GEN_IP_PKTINFO, &one, sizeof(one));     // linux supports this, so why not - might fail on other systems
-#ifdef IPV6_RECVPKTINFO
-      setsockopt(s, IPPROTO_IPV6, IPV6_RECVPKTINFO, &one, sizeof(one)); 
-#endif
-      setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, &one, sizeof(one));      // if this fails, we report an error in tcpreceiver too
-    }
-
-    if (!setSocketTimestamps(s))
-      g_log<<Logger::Warning<<"Unable to enable timestamp reporting for socket"<<endl;
-
-#ifdef SO_REUSEPORT
-    if( d_can_reuseport )
-        if( setsockopt(s, SOL_SOCKET, SO_REUSEPORT, &one, sizeof(one)) )
-          d_can_reuseport = false;
-#endif
-
-    if( ::arg().mustDo("non-local-bind") )
-	Utility::setBindAny(AF_INET6, s);
-
-    if( !d_additional_socket )
-        g_localaddresses.push_back(locala);
-    if(::bind(s, (sockaddr*)&locala, locala.getSocklen())<0) {
-      close(s);
-      if( errno == EADDRNOTAVAIL && ! ::arg().mustDo("local-ipv6-nonexist-fail") ) {
-        g_log<<Logger::Error<<"IPv6 Address " << localname << " does not exist on this server - skipping UDP bind" << endl;
-        continue;
-      } else {
-        g_log<<Logger::Error<<"Unable to bind to UDPv6 socket "<< localname <<": "<<strerror(errno)<<endl;
-        throw PDNSException("Unable to bind to UDPv6 socket");
-      }
-    }
-    d_sockets.push_back(s);
-    struct pollfd pfd;
-    pfd.fd = s;
-    pfd.events = POLLIN;
-    pfd.revents = 0;
-    d_rfds.push_back(pfd);
-    g_log<<Logger::Error<<"UDPv6 server bound to "<<locala.toStringWithPort()<<endl;
-  }
-}
-
 UDPNameserver::UDPNameserver( bool additional_socket )
 {
 #ifdef SO_REUSEPORT
@@ -279,39 +214,36 @@ UDPNameserver::UDPNameserver( bool additional_socket )
   // Are we the main socket (false) or a rebinding using SO_REUSEPORT ?
   d_additional_socket = additional_socket;
 
-  if(!::arg()["local-address"].empty())
-    bindIPv4();
-  if(!::arg()["local-ipv6"].empty())
-    bindIPv6();
-
-  if(::arg()["local-address"].empty() && ::arg()["local-ipv6"].empty()) 
+  if(::arg()["local-address"].empty())
     g_log<<Logger::Critical<<"PDNS is deaf and mute! Not listening on any interfaces"<<endl;    
+
+  bindAddresses();
 }
 
-void UDPNameserver::send(DNSPacket *p)
+void UDPNameserver::send(DNSPacket& p)
 {
-  string buffer=p->getString();
-  g_rs.submitResponse(*p, true);
+  const string& buffer=p.getString();
+  g_rs.submitResponse(p, true);
 
   struct msghdr msgh;
   struct iovec iov;
-  char cbuf[256];
+  cmsgbuf_aligned cbuf;
 
-  fillMSGHdr(&msgh, &iov, cbuf, 0, (char*)buffer.c_str(), buffer.length(), &p->d_remote);
+  fillMSGHdr(&msgh, &iov, &cbuf, 0, (char*)buffer.c_str(), buffer.length(), &p.d_remote);
 
   msgh.msg_control=NULL;
-  if(p->d_anyLocal) {
-    addCMsgSrcAddr(&msgh, cbuf, p->d_anyLocal.get_ptr(), 0);
+  if(p.d_anyLocal) {
+    addCMsgSrcAddr(&msgh, &cbuf, p.d_anyLocal.get_ptr(), 0);
   }
-  DLOG(g_log<<Logger::Notice<<"Sending a packet to "<< p->getRemote() <<" ("<< buffer.length()<<" octets)"<<endl);
-  if(buffer.length() > p->getMaxReplyLen()) {
-    g_log<<Logger::Error<<"Weird, trying to send a message that needs truncation, "<< buffer.length()<<" > "<<p->getMaxReplyLen()<<". Question was for "<<p->qdomain<<"|"<<p->qtype.getName()<<endl;
+  DLOG(g_log<<Logger::Notice<<"Sending a packet to "<< p.getRemote() <<" ("<< buffer.length()<<" octets)"<<endl);
+  if(buffer.length() > p.getMaxReplyLen()) {
+    g_log<<Logger::Error<<"Weird, trying to send a message that needs truncation, "<< buffer.length()<<" > "<<p.getMaxReplyLen()<<". Question was for "<<p.qdomain<<"|"<<p.qtype.getName()<<endl;
   }
-  if(sendmsg(p->getSocket(), &msgh, 0) < 0)
-    g_log<<Logger::Error<<"Error sending reply with sendmsg (socket="<<p->getSocket()<<", dest="<<p->d_remote.toStringWithPort()<<"): "<<strerror(errno)<<endl;
+  if(sendmsg(p.getSocket(), &msgh, 0) < 0)
+    g_log<<Logger::Error<<"Error sending reply with sendmsg (socket="<<p.getSocket()<<", dest="<<p.d_remote.toStringWithPort()<<"): "<<stringerror()<<endl;
 }
 
-DNSPacket *UDPNameserver::receive(DNSPacket *prefilled, std::string& buffer)
+bool UDPNameserver::receive(DNSPacket& packet, std::string& buffer)
 {
   ComboAddress remote;
   extern StatBag S;
@@ -320,10 +252,10 @@ DNSPacket *UDPNameserver::receive(DNSPacket *prefilled, std::string& buffer)
 
   struct msghdr msgh;
   struct iovec iov;
-  char cbuf[256];
+  cmsgbuf_aligned cbuf;
 
   remote.sin6.sin6_family=AF_INET6; // make sure it is big enough
-  fillMSGHdr(&msgh, &iov, cbuf, sizeof(cbuf), &buffer.at(0), buffer.size(), &remote);
+  fillMSGHdr(&msgh, &iov, &cbuf, sizeof(cbuf), &buffer.at(0), buffer.size(), &remote);
   
   int err;
   vector<struct pollfd> rfds= d_rfds;
@@ -347,7 +279,7 @@ DNSPacket *UDPNameserver::receive(DNSPacket *prefilled, std::string& buffer)
       sock=pfd.fd;        
       if((len=recvmsg(sock, &msgh, 0)) < 0 ) {
         if(errno != EAGAIN)
-          g_log<<Logger::Error<<"recvfrom gave error, ignoring: "<<strerror(errno)<<endl;
+          g_log<<Logger::Error<<"recvfrom gave error, ignoring: "<<stringerror()<<endl;
         return 0;
       }
       break;
@@ -363,36 +295,28 @@ DNSPacket *UDPNameserver::receive(DNSPacket *prefilled, std::string& buffer)
   if(remote.sin4.sin_port == 0) // would generate error on responding. sin4 also works for ipv6
     return 0;
   
-  DNSPacket *packet;
-  if(prefilled)  // they gave us a preallocated packet
-    packet=prefilled;
-  else
-    packet=new DNSPacket(true); // don't forget to free it!
-
-  packet->setSocket(sock);
-  packet->setRemote(&remote);
+  packet.setSocket(sock);
+  packet.setRemote(&remote);
 
   ComboAddress dest;
   if(HarvestDestinationAddress(&msgh, &dest)) {
 //    cerr<<"Setting d_anyLocal to '"<<dest.toString()<<"'"<<endl;
-    packet->d_anyLocal = dest;
+    packet.d_anyLocal = dest;
   }            
 
   struct timeval recvtv;
   if(HarvestTimestamp(&msgh, &recvtv)) {
-    packet->d_dt.setTimeval(recvtv);
+    packet.d_dt.setTimeval(recvtv);
   }
   else
-    packet->d_dt.set(); // timing    
+    packet.d_dt.set(); // timing    
 
-  if(packet->parse(&buffer.at(0), (size_t) len)<0) {
+  if(packet.parse(&buffer.at(0), (size_t) len)<0) {
     S.inc("corrupt-packets");
-    S.ringAccount("remotes-corrupt", packet->d_remote);
+    S.ringAccount("remotes-corrupt", packet.d_remote);
 
-    if(!prefilled)
-      delete packet;
-    return 0; // unable to parse
+    return false; // unable to parse
   }
   
-  return packet;
+  return true;
 }
