@@ -142,18 +142,17 @@ try
     throw PDNSException("tcp read failed");
 
   len=ntohs(len);
-  char *creply = new char[len];
+  std::unique_ptr<char[]> creply(new char[len]);
   int n=0;
   int numread;
   while(n<len) {
-    numread=sock.read(creply+n, len-n);
+    numread=sock.read(creply.get()+n, len-n);
     if(numread<0)
       throw PDNSException("tcp read failed");
     n+=numread;
   }
 
-  string reply(creply, len);
-  delete[] creply;
+  string reply(creply.get(), len);
 
   MOADNSParser mdp(false, reply);
   cout<<"Reply to question for qname='"<<mdp.d_qname<<"', qtype="<<DNSRecordContent::NumberToType(mdp.d_qtype)<<endl;
