@@ -33,7 +33,7 @@ BOOST_AUTO_TEST_CASE(test_parser) {
         BP.parse(pathbuf.str());
 
         vector<BindDomainInfo> domains=BP.getDomains();
-        BOOST_CHECK_EQUAL(domains.size(), 11);
+        BOOST_CHECK_EQUAL(domains.size(), 11U);
 
 #define checkzone(i, dname, fname, ztype, nmasters) { \
 	        BOOST_CHECK(domains[i].name == DNSName(#dname));     \
@@ -42,18 +42,18 @@ BOOST_AUTO_TEST_CASE(test_parser) {
                 BOOST_CHECK_EQUAL(domains[i].masters.size(), nmasters); \
         }
 
-        checkzone(0, example.com, "./zones//example.com", master, 0);
-        checkzone(1, test.com, "./zones//test.com", slave, 1);
+        checkzone(0, example.com, "./zones//example.com", master, 0U);
+        checkzone(1, test.com, "./zones//test.com", slave, 1U);
         BOOST_CHECK_EQUAL(domains[1].masters[0].toString(), ComboAddress("1.2.3.4", 5678).toString());
-        checkzone(2, test.dyndns, "./zones//test.dyndns", garblewarble, 0);
-        checkzone(3, wtest.com, "./zones//wtest.com", master, 0);
-        checkzone(4, nztest.com, "./zones//nztest.com", master, 0);
-        checkzone(5, dnssec-parent.com, "./zones//dnssec-parent.com", master, 0);
-        checkzone(6, delegated.dnssec-parent.com, "./zones//delegated.dnssec-parent.com", master, 0);
-        checkzone(7, secure-delegated.dnssec-parent.com, "./zones//secure-delegated.dnssec-parent.com", master, 0);
-        checkzone(8, minimal.com, "./zones//minimal.com", master, 0);
-        checkzone(9, tsig.com, "./zones//tsig.com", master, 0);
-        checkzone(10, stest.com, "./zones//stest.com", master, 0);
+        checkzone(2, test.dyndns, "./zones//test.dyndns", garblewarble, 0U);
+        checkzone(3, wtest.com, "./zones//wtest.com", master, 0U);
+        checkzone(4, nztest.com, "./zones//nztest.com", master, 0U);
+        checkzone(5, dnssec-parent.com, "./zones//dnssec-parent.com", master, 0U);
+        checkzone(6, delegated.dnssec-parent.com, "./zones//delegated.dnssec-parent.com", master, 0U);
+        checkzone(7, secure-delegated.dnssec-parent.com, "./zones//secure-delegated.dnssec-parent.com", master, 0U);
+        checkzone(8, minimal.com, "./zones//minimal.com", master, 0U);
+        checkzone(9, tsig.com, "./zones//tsig.com", master, 0U);
+        checkzone(10, stest.com, "./zones//stest.com", master, 0U);
 }
 
 BOOST_AUTO_TEST_SUITE_END()
