@@ -15,7 +15,7 @@
 
 using std::string;
 
-BOOST_AUTO_TEST_SUITE(misc_hh)
+BOOST_AUTO_TEST_SUITE(test_misc_hh)
 typedef pair<std::string, uint16_t> typedns_t;
 
 BOOST_AUTO_TEST_CASE(test_CIStringCompare) {
@@ -26,7 +26,7 @@ BOOST_AUTO_TEST_CASE(test_CIStringCompare) {
         nsset.insert("def");
         nsset.insert("aBc");
         nsset.insert("ns.example.com");
-        BOOST_CHECK_EQUAL(nsset.size(), 4);
+        BOOST_CHECK_EQUAL(nsset.size(), 4U);
 
         ostringstream s;
         for(set<std::string, CIStringCompare>::const_iterator i=nsset.begin();i!=nsset.end();++i) {
@@ -46,7 +46,7 @@ BOOST_AUTO_TEST_CASE(test_CIStringPairCompare) {
         nsset2.insert(make_pair("ns.example.com", 0));
         nsset2.insert(make_pair("abc", 2));
         nsset2.insert(make_pair("ABC", 2));
-        BOOST_CHECK_EQUAL(nsset2.size(), 6);
+        BOOST_CHECK_EQUAL(nsset2.size(), 6U);
 
         ostringstream s;
         for(set<typedns_t, CIStringPairCompare>::const_iterator i=nsset2.begin();i!=nsset2.end();++i) {
@@ -109,19 +109,15 @@ BOOST_AUTO_TEST_CASE(test_stripDot) {
 }
 
 BOOST_AUTO_TEST_CASE(test_labelReverse) {
-    BOOST_CHECK_EQUAL(labelReverse("www.powerdns.com"), "com powerdns www");
+  BOOST_CHECK_EQUAL(DNSName("www.powerdns.com").labelReverse().toString(" ", false), "com powerdns www");
 }
 
-BOOST_AUTO_TEST_CASE(test_makeRelative) {
-    BOOST_CHECK_EQUAL(makeRelative("www.powerdns.com", "powerdns.com"), "www");
-    BOOST_CHECK_EQUAL(makeRelative("PoWeRdNs.CoM", "powerdns.com"), "");
-}
 
 BOOST_AUTO_TEST_CASE(test_AtomicCounter) {
     AtomicCounter ac(0);
     ++ac;
     ++ac;
-    BOOST_CHECK_EQUAL(ac, 2);
+    BOOST_CHECK_EQUAL(ac, 2U);
 }
 
 BOOST_AUTO_TEST_CASE(test_endianness) {
@@ -180,5 +176,31 @@ BOOST_AUTO_TEST_CASE(test_SimpleMatch) {
   BOOST_CHECK_EQUAL(SimpleMatch("abc*").match(std::string("abc")), true);
 }
 
+template<typename T> bool rfc1982check(T x, T y) {
+  return rfc1982LessThan(x, y);
+}
+
+BOOST_AUTO_TEST_CASE(test_rfc1982LessThan) {
+  // The test cases from rfc1982 section 5.2
+  BOOST_CHECK(rfc1982check<uint8_t>(0, 1));
+  BOOST_CHECK(rfc1982check<uint8_t>(0, 44));
+  BOOST_CHECK(rfc1982check<uint8_t>(0, 100));
+  BOOST_CHECK(rfc1982check<uint8_t>(44, 100));
+  BOOST_CHECK(rfc1982check<uint8_t>(100, 200));
+  BOOST_CHECK(rfc1982check<uint8_t>(200, 255));
+  BOOST_CHECK(rfc1982check<uint8_t>(255, 0));
+  BOOST_CHECK(rfc1982check<uint8_t>(255, 100));
+  BOOST_CHECK(rfc1982check<uint8_t>(200, 0));
+  BOOST_CHECK(rfc1982check<uint8_t>(200, 44));
+
+  BOOST_CHECK(rfc1982check<uint32_t>(0, 1));
+  BOOST_CHECK(rfc1982check<uint32_t>(UINT32_MAX-10, 1));
+  BOOST_CHECK(rfc1982check<uint32_t>(UINT32_MAX/2, UINT32_MAX-10));
+
+  BOOST_CHECK(rfc1982check<uint64_t>(0, 1));
+  BOOST_CHECK(rfc1982check<uint64_t>(UINT64_MAX-10, 1));
+  BOOST_CHECK(rfc1982check<uint64_t>(UINT64_MAX/2, UINT64_MAX-10));
+}
+
 BOOST_AUTO_TEST_SUITE_END()
 
