@@ -50,7 +50,7 @@
 
 StatBag S;
 
-enum dbmode_t {MYSQL, GORACLE, POSTGRES, SQLITE, MYDNS, ORACLE};
+enum dbmode_t {MYSQL, POSTGRES, SQLITE};
 static dbmode_t g_mode;
 static bool g_intransaction;
 static int g_numRecords;
@@ -71,7 +71,7 @@ static string stripDotContent(const string& content)
 
 static string sqlstr(const string &name)
 {
-  if(g_mode == SQLITE || g_mode==GORACLE || g_mode==ORACLE)
+  if(g_mode == SQLITE)
     return "'"+boost::replace_all_copy(name, "'", "''")+"'";
   
   string a;
@@ -99,15 +99,15 @@ static void startNewTransaction()
     if(g_mode==POSTGRES) {
       cout<<"COMMIT WORK;"<<endl;
     }
-    else if(g_mode == MYSQL || g_mode == SQLITE || g_mode == MYDNS) {
+    else if(g_mode == MYSQL || g_mode == SQLITE) {
       cout<<"COMMIT;"<<endl;
     }
   }
   g_intransaction=1;
   
-  if(g_mode == MYSQL || g_mode == MYDNS)
+  if(g_mode == MYSQL)
     cout<<"BEGIN;"<<endl;
-  else if (g_mode!=GORACLE && g_mode!=ORACLE)
+  else
     cout<<"BEGIN TRANSACTION;"<<endl;
 }
 
@@ -117,12 +117,6 @@ static void emitDomain(const DNSName& domain, const vector<ComboAddress> *master
     if(g_mode==POSTGRES || g_mode==MYSQL || g_mode==SQLITE) {
       cout<<"insert into domains (name,type) values ("<<toLower(sqlstr(iDomain))<<",'NATIVE');"<<endl;
     }
-    else if(g_mode==GORACLE) {
-      cout<<"insert into domains (id,name,type) values (domains_id_sequence.nextval,"<<toLower(sqlstr(iDomain))<<",'NATIVE');"<<endl;
-    }
-    else if(g_mode==ORACLE) {
-      cout<<"INSERT INTO Zones (id, name, type) VALUES (zones_id_seq.nextval, "<<sqlstr(toLower(iDomain))<<", 'NATIVE');"<<endl;
-    }
   }
   else 
   {
@@ -140,9 +134,6 @@ static void emitDomain(const DNSName& domain, const vector<ComboAddress> *master
       else
         cout<<"insert into domains (name,type,master) values ("<<sqlstr(iDomain)<<",'SLAVE'"<<", '"<<mstrs<<"');"<<endl;
     }
-    else if (g_mode == GORACLE || g_mode==ORACLE) {
-      cerr<<"Slave import mode not supported with oracle."<<endl;
-    }
   }
 }
 
@@ -175,7 +166,7 @@ static void emitRecord(const DNSName& zoneName, const DNSName &DNSqname, const s
   if(qtype == "NSEC" || qtype == "NSEC3")
     return; // NSECs do not go in the database
 
-  if((qtype == "MX" || qtype == "SRV") && g_mode!=ORACLE) {
+  if((qtype == "MX" || qtype == "SRV")) {
     prio=pdns_stou(content);
     
     string::size_type pos = content.find_first_not_of("0123456789");
@@ -208,52 +199,11 @@ static void emitRecord(const DNSName& zoneName, const DNSName &DNSqname, const s
       sqlstr(stripDotContent(content))<<", "<<ttl<<", "<<prio<<", '"<<(disabled ? 't': 'f') <<
       "' from domains where name="<<toLower(sqlstr(zname))<<";\n";
   }
-  else if(g_mode==GORACLE) {
-    cout<<"insert into Records (id, domain_id, name, type, content, ttl, prio, disabled) select RECORDS_ID_SEQUENCE.nextval,id ,"<<
-      sqlstr(toLower(qname))<<", "<<
-      sqlstr(qtype)<<", "<<
-      sqlstr(stripDotContent(content))<<", "<<ttl<<", "<<prio<<", "<<disabled<<
-      " from Domains where name="<<toLower(sqlstr(zname))<<";\n";
-  }
-  else if(g_mode==ORACLE) {
-    cout<<"INSERT INTO Records (id, zone_id, fqdn, ttl, type, content) SELECT records_id_seq.nextval, id, "<<
-      sqlstr(toLower(qname))<<", "<<
-      ttl<<", "<<sqlstr(qtype)<<", "<<
-      sqlstr(stripDotContent(content))<<
-      " FROM Zones WHERE name="<<toLower(sqlstr(zname))<<";"<<endl;
-  }
-  else if (g_mode == MYDNS) {
-    string zoneNameDot = zname + ".";
-    if (qtype == "A" || qtype == "AAAA" || qtype == "CNAME" || qtype == "HINFO" || qtype == "MX" || qtype == "NAPTR" || 
-        qtype == "NS" || qtype == "PTR" || qtype == "RP" || qtype == "SRV" || qtype == "TXT")
-    {
-      if ((qtype == "MX" || qtype == "NS" || qtype == "SRV" || qtype == "CNAME") && content[content.size()-1] != '.')
-        content.append(".");
-      cout<<"INSERT INTO rr(zone, name, type, data, aux, ttl) VALUES("<<
-      "(SELECT id FROM soa WHERE origin = "<< 
-      sqlstr(toLower(zoneNameDot))<<"), "<<
-      sqlstr(toLower(DNSqname.toString()))<<", "<<
-      sqlstr(qtype)<<", "<<sqlstr(content)<<", "<<prio<<", "<<ttl<<");\n";
-    }
-    else if (qtype == "SOA") {
-      //pdns CONTENT = ns1.wtest.com. ahu.example.com. 2005092501 28800 7200 604800 86400 
-      vector<string> parts;
-      stringtok(parts, content);
- 
-      cout<<"INSERT INTO soa(origin, ns, mbox, serial, refresh, retry, expire, minimum, ttl) VALUES("<<
-      sqlstr(toLower(zoneNameDot))<<", "<<sqlstr(parts[0])<<", "<<sqlstr(parts[1])<<", "<<pdns_stou(parts[2])<<", "<<
-      pdns_stou(parts[3])<<", "<<pdns_stou(parts[4])<<", "<<pdns_stou(parts[5])<<", "<<pdns_stou(parts[6])<<", "<<ttl<<");\n";
-    }
-    else
-    {
-      cerr<<"Record type "<<qtype<<" is not supported."<<endl;
-    }
-  }
 }
 
 
 /* 2 modes of operation, either --named or --zone (the latter needs $ORIGIN) 
-   2 further modes: --mysql, --goracle or --oracle
+   1 further mode: --mysql
 */
 
 ArgvMap &arg()
@@ -271,9 +221,6 @@ try
   
     ::arg().setSwitch("gpgsql","Output in format suitable for default gpgsqlbackend")="no";
     ::arg().setSwitch("gmysql","Output in format suitable for default gmysqlbackend")="no";
-    ::arg().setSwitch("mydns","Output in format suitable for default mydnsbackend")="no";
-    ::arg().setSwitch("goracle","Output in format suitable for the goraclebackend")="no";
-    ::arg().setSwitch("oracle","Output in format suitable for the oraclebackend")="no";
     ::arg().setSwitch("gsqlite","Output in format suitable for default gsqlitebackend")="no";
     ::arg().setSwitch("verbose","Verbose comments on operation")="no";
     ::arg().setSwitch("slave","Keep BIND slaves as slaves. Only works with named-conf.")="no";
@@ -289,6 +236,7 @@ try
     ::arg().set("soa-refresh-default","Do not change")="0";
     ::arg().set("soa-retry-default","Do not change")="0";
     ::arg().set("soa-expire-default","Do not change")="0";
+    ::arg().set("max-generate-steps", "Maximum number of $GENERATE steps when loading a zone from a file")="0";
 
     ::arg().setCmd("help","Provide a helpful message");
     ::arg().setCmd("version","Print the version");
@@ -327,12 +275,6 @@ try
       g_mode=POSTGRES;
     else if(::arg().mustDo("gsqlite"))
       g_mode=SQLITE;
-    else if(::arg().mustDo("goracle"))
-      g_mode=GORACLE;
-    else if(::arg().mustDo("oracle"))
-      g_mode=ORACLE;
-    else if(::arg().mustDo("mydns"))
-      g_mode=MYDNS;
     else {
       cerr<<"Unknown SQL mode!\n\n";
       cerr<<"syntax:"<<endl<<endl;
@@ -340,9 +282,6 @@ try
       exit(1);
     }
 
-    if((g_mode==GORACLE || g_mode==ORACLE) && !::arg().mustDo("transactions"))
-      cout<<"set autocommit on;"<<endl;
-
     namedfile=::arg()["named-conf"];
     zonefile=::arg()["zone"];
 
@@ -381,6 +320,7 @@ try
             emitDomain(i->name, &(i->masters));
             
             ZoneParserTNG zpt(i->filename, i->name, BP.getDirectory());
+            zpt.setMaxGenerateSteps(::arg().asNum("max-generate-steps"));
             DNSResourceRecord rr;
             bool seenSOA=false;
             string comment;
@@ -419,6 +359,7 @@ try
         zonename = DNSName(::arg()["zone-name"]);
 
       ZoneParserTNG zpt(zonefile, zonename);
+      zpt.setMaxGenerateSteps(::arg().asNum("max-generate-steps"));
       DNSResourceRecord rr;
       startNewTransaction();
       string comment;
